import { Node } from "./ast/node";
import { SourceLocation } from "./ast/source-location";
import { Thing } from "./thing";
import { Expression } from "./expression";
import { ParseError } from "./error";
export declare const enum ErrorCategory {
    SyntaxError = "SyntaxError",
    SemanticError = "SemanticError",
    Warning = "Warning"
}
export interface Error {
    code: number;
    name: string;
    category: ErrorCategory;
}
export type LocationObject = Expression | Node | Thing;
/**
 * Get the first expression in @param expressions that has a truthy `loc` field, or fallback to the last expression provided. Used to get expression when calling error factories.
 */
export declare const getExpressionIfLocField: (...expressions: (LocationObject | undefined)[]) => LocationObject | undefined;
/**
 * Should only be used for validation errors on non-Node/Expression/Thing
 */
export type ExpressionLocationInfo = {
    loc?: SourceLocation;
    uri?: string;
};
export type ValidationErrorFactoryInput = {
    expr?: Expression | Node | Thing | ExpressionLocationInfo;
    attributes?: Record<string, string | number | undefined>;
    overrideMessage?: string;
};
type ValidationErrorFactoryFunction = (input?: ValidationErrorFactoryInput) => ParseError<Error, string>;
export declare const ValidationErrorFactory: {
    readonly SyntaxError: ValidationErrorFactoryFunction;
    /**
     * Unexpected operator.
     *
     * Supports:
     * `!`, `&&`, `||`, `<`, `<=`, `=`, `>`, `>=`.
     */
    readonly UnexpectedOperator: ValidationErrorFactoryFunction;
    readonly UnexpectedOperatorWithText: ValidationErrorFactoryFunction;
    readonly UnexpectedBooleanOperator: ValidationErrorFactoryFunction;
    readonly MissingUtteranceClosingBrace: ValidationErrorFactoryFunction;
    readonly MissingUtteranceOpeningBrace: ValidationErrorFactoryFunction;
    readonly EmptyUtteranceBraces: ValidationErrorFactoryFunction;
    readonly InvalidImport: ValidationErrorFactoryFunction;
    readonly InvalidImportNamespace: ValidationErrorFactoryFunction;
    readonly TypeError: ValidationErrorFactoryFunction;
    readonly TypeErrorDialogLastExpr: ValidationErrorFactoryFunction;
    readonly TypeErrorRequiredTypeArguments: ValidationErrorFactoryFunction;
    readonly TypeErrorListType: ValidationErrorFactoryFunction;
    readonly TypeErrorListItem: ValidationErrorFactoryFunction;
    readonly TypeErrorIncomparableExpr: ValidationErrorFactoryFunction;
    readonly TypeErrorWhenBoolean: ValidationErrorFactoryFunction;
    readonly UnexpectedBooleanExpression: ValidationErrorFactoryFunction;
    readonly UnknownProperty: ValidationErrorFactoryFunction;
    readonly UnknownPropertyIndex: ValidationErrorFactoryFunction;
    readonly NameNotFound: ValidationErrorFactoryFunction;
    readonly DeclarationExpected: ValidationErrorFactoryFunction;
    readonly MissingArguments: ValidationErrorFactoryFunction;
    readonly UnknownArgument: ValidationErrorFactoryFunction;
    readonly UnknownArgumentIndex: ValidationErrorFactoryFunction;
    readonly DuplicateArgument: ValidationErrorFactoryFunction;
    readonly MissingProperties: ValidationErrorFactoryFunction;
    readonly MissingPropertiesOnType: ValidationErrorFactoryFunction;
    readonly MissingPropertiesOnUtteranceType: ValidationErrorFactoryFunction;
    readonly CircularReferenceDialog: ValidationErrorFactoryFunction;
    readonly CircularReferenceName: ValidationErrorFactoryFunction;
    readonly CircularReferenceType: ValidationErrorFactoryFunction;
    readonly FileNotFound: ValidationErrorFactoryFunction;
    readonly FileReadError: ValidationErrorFactoryFunction;
    readonly InvalidFileTypeApla: ValidationErrorFactoryFunction;
    readonly InvalidFileTypeApl: ValidationErrorFactoryFunction;
    readonly PathNotAccessible: ValidationErrorFactoryFunction;
    readonly TooLargeIR: ValidationErrorFactoryFunction;
    readonly InvalidCallContextModuleNameDeclaration: ValidationErrorFactoryFunction;
    readonly InvalidCallContextInModule: ValidationErrorFactoryFunction;
    readonly InvalidCallContextInCondition: ValidationErrorFactoryFunction;
    readonly InvalidCallContextComparison: ValidationErrorFactoryFunction;
    readonly InvalidCallContextType: ValidationErrorFactoryFunction;
    readonly InvalidCallContextTypeInline: ValidationErrorFactoryFunction;
    readonly InvalidAnnotationTarget: ValidationErrorFactoryFunction;
    readonly InvalidAnnotationMultiple: ValidationErrorFactoryFunction;
    readonly InvalidAnnotationCall: ValidationErrorFactoryFunction;
    readonly InvalidAnnotationNameDeclarationVariations: ValidationErrorFactoryFunction;
    readonly InvalidAnnotationNameDeclarationNameReference: ValidationErrorFactoryFunction;
    readonly InvalidAnnotationReusableDialog: ValidationErrorFactoryFunction;
    readonly InvalidAnnotationReusableDialogSample: ValidationErrorFactoryFunction;
    readonly InvalidAnnotationMetaAnnotation: ValidationErrorFactoryFunction;
    readonly InvalidAnnotationMetaAnnotationReturnType: ValidationErrorFactoryFunction;
    readonly InvalidAnnotationValidateArg: ValidationErrorFactoryFunction;
    readonly InvalidAnnotationValidateArgCondition: ValidationErrorFactoryFunction;
    readonly ConditionWhenMustCompareApiResponse: ValidationErrorFactoryFunction;
    readonly EmptyConditionBlock: ValidationErrorFactoryFunction;
    readonly InvalidFirstExprInConditionBlock: ValidationErrorFactoryFunction;
    readonly InvalidFirstResponseInConditionBlock: ValidationErrorFactoryFunction;
    readonly ExprRequiredInConditionBlock: ValidationErrorFactoryFunction;
    readonly InvalidDialogActArgument: ValidationErrorFactoryFunction;
    readonly MissingAffirmORDenyEventAfterConfirmAction: ValidationErrorFactoryFunction;
    readonly MissingApiInvocationAfterConfirmAction: ValidationErrorFactoryFunction;
    readonly MissingApiInvocationAfterConfirmArgs: ValidationErrorFactoryFunction;
    readonly MissingAffirmORDenyEventAfterConfirmArgs: ValidationErrorFactoryFunction;
    readonly MissingAffirmORDenyEventAfterConfirmArgsBeforeCall: ValidationErrorFactoryFunction;
    readonly MissingAffirmORDenyEventAfterConfirmArgsBeforeExpect: ValidationErrorFactoryFunction;
    readonly InvalidNotifyActionNameUnusedAction: ValidationErrorFactoryFunction;
    readonly InvalidNotifyActionNameLastAction: ValidationErrorFactoryFunction;
    readonly InvalidNotifyActionNameAlreadyNotified: ValidationErrorFactoryFunction;
    readonly InvalidNextAct: ValidationErrorFactoryFunction;
    readonly InvalidNextActNotify: ValidationErrorFactoryFunction;
    readonly InvalidReqAltFirstAct: ValidationErrorFactoryFunction;
    readonly InvalidReqAltNextAct: ValidationErrorFactoryFunction;
    readonly MissingRequestPromptForRequiredApiArgument: ValidationErrorFactoryFunction;
    readonly MissingRequestPromptForMultipleArguments: ValidationErrorFactoryFunction;
    readonly MissingRequestPromptForConfirmArgs: ValidationErrorFactoryFunction;
    readonly MissingExpectInvokeEvent: ValidationErrorFactoryFunction;
    readonly InvalidWelcomeAction: ValidationErrorFactoryFunction;
    readonly InvalidWelcomeActionReturnValue: ValidationErrorFactoryFunction;
    readonly TooManyWelcomeCalls: ValidationErrorFactoryFunction;
    readonly InvalidLoadContextMultipleCall: ValidationErrorFactoryFunction;
    readonly InvalidLoadContextNotFirstCall: ValidationErrorFactoryFunction;
    readonly InvalidLoadContextActionArguments: ValidationErrorFactoryFunction;
    readonly InvalidLoadContextActionGenericArguments: ValidationErrorFactoryFunction;
    readonly InvalidLoadContextActionReturnType: ValidationErrorFactoryFunction;
    readonly InvalidLoadContextActionNoActionReference: ValidationErrorFactoryFunction;
    readonly InvalidSaveContextActionGenericArguments: ValidationErrorFactoryFunction;
    readonly InvalidSaveContextActionReturnType: ValidationErrorFactoryFunction;
    readonly InvalidSaveContextActionArgumentType: ValidationErrorFactoryFunction;
    readonly InvalidSaveContextActionNoActionReference: ValidationErrorFactoryFunction;
    readonly InvalidTrackActionsEmptyList: ValidationErrorFactoryFunction;
    readonly InvalidTrackActionsLoadContext: ValidationErrorFactoryFunction;
    readonly InvalidTrackActionsSaveContext: ValidationErrorFactoryFunction;
    readonly InvalidTrackActionsBuiltIn: ValidationErrorFactoryFunction;
    readonly InvalidEvent: ValidationErrorFactoryFunction;
    readonly InvalidEventInvoke: ValidationErrorFactoryFunction;
    readonly InvalidEventAffirmOrDeny: ValidationErrorFactoryFunction;
    readonly InvalidEventInform: ValidationErrorFactoryFunction;
    readonly InvalidEventFollowingConfirmAction: ValidationErrorFactoryFunction;
    readonly InvalidEventResponse: ValidationErrorFactoryFunction;
    readonly InvalidFirstEvent: ValidationErrorFactoryFunction;
    readonly InvalidLocaleInSampleDialog: ValidationErrorFactoryFunction;
    readonly InvalidLocaleInSampleAction: ValidationErrorFactoryFunction;
    readonly InvalidLocaleInSampleType: ValidationErrorFactoryFunction;
    readonly InvalidLocaleInSampleNameOrProperty: ValidationErrorFactoryFunction;
    readonly MissingLocale: ValidationErrorFactoryFunction;
    readonly DuplicateLocale: ValidationErrorFactoryFunction;
    readonly MissingSampleForLocale: ValidationErrorFactoryFunction;
    readonly MissingSkillResponseForLocale: ValidationErrorFactoryFunction;
    readonly MissingInteractionModelForLocale: ValidationErrorFactoryFunction;
    readonly MissingSlotTypeForLocale: ValidationErrorFactoryFunction;
    readonly MissingSlotTypeForLocaleTypeDependency: ValidationErrorFactoryFunction;
    readonly InvalidPayloadType: ValidationErrorFactoryFunction;
    readonly InvalidPayloadPropertyValue: ValidationErrorFactoryFunction;
    readonly MismatchedPayloadPropertyValueActionArgument: ValidationErrorFactoryFunction;
    readonly MismatchedPayloadPropertyValuePreviouslyConfirmed: ValidationErrorFactoryFunction;
    readonly MismatchedConfirmArgsPayloadProperty: ValidationErrorFactoryFunction;
    readonly MissingConfirmedArgsInPayload: ValidationErrorFactoryFunction;
    readonly IllegalNothingValueBoolean: ValidationErrorFactoryFunction;
    readonly IllegalNothingValueResponsePrompt: ValidationErrorFactoryFunction;
    readonly IllegalNothingValueActionArgument: ValidationErrorFactoryFunction;
    readonly InvalidApiArgument: ValidationErrorFactoryFunction;
    readonly InvalidApiArgumentDefaultValue: ValidationErrorFactoryFunction;
    readonly InvalidApiArgumentRedeclare: ValidationErrorFactoryFunction;
    readonly MismatchedActionArgumentsValidateArguments: ValidationErrorFactoryFunction;
    readonly MismatchedActionArgumentsListValue: ValidationErrorFactoryFunction;
    readonly MismatchedActionCarryOverArguments: ValidationErrorFactoryFunction;
    readonly MismatchedActionArgumentsList: ValidationErrorFactoryFunction;
    readonly ExpectedLiteralValue: ValidationErrorFactoryFunction;
    readonly ExpectedNonEmptyList: ValidationErrorFactoryFunction;
    readonly InvalidEnsureArgOrder: ValidationErrorFactoryFunction;
    readonly InvalidEnsureArgOptional: ValidationErrorFactoryFunction;
    readonly InvalidAplaDocumentName: ValidationErrorFactoryFunction;
    readonly InvalidAplDocumentName: ValidationErrorFactoryFunction;
    readonly InvalidVariationsArgumentSkillLevelResponses: ValidationErrorFactoryFunction;
    readonly InvalidVariationsArgumentResponsePrompts: ValidationErrorFactoryFunction;
    readonly InvalidPolicy: ValidationErrorFactoryFunction;
    readonly InvalidPolicyWithPolicyName: ValidationErrorFactoryFunction;
    readonly ResetArgument: ValidationErrorFactoryFunction;
    readonly InvalidBooleanExpressionTypeLiteral: ValidationErrorFactoryFunction;
    readonly InvalidBooleanExpressionAction: ValidationErrorFactoryFunction;
    readonly InvalidRequestAct: ValidationErrorFactoryFunction;
    readonly InformUtteranceSampleMissingSlot: ValidationErrorFactoryFunction;
    readonly MismatchedArgsInReqAlt: ValidationErrorFactoryFunction;
    readonly MissingAplaInResponse: ValidationErrorFactoryFunction;
    readonly InvalidLastExpressionInSample: ValidationErrorFactoryFunction;
    readonly InvalidLastExpressionInSampleFromDialogCall: ValidationErrorFactoryFunction;
    readonly MissingRequiredPrompts: ValidationErrorFactoryFunction;
    readonly EmptyEnumDeclaration: ValidationErrorFactoryFunction;
    readonly DuplicateEnumItem: ValidationErrorFactoryFunction;
    readonly DuplicateNameDeclaration: ValidationErrorFactoryFunction;
    readonly DuplicateNameDeclarationInModule: ValidationErrorFactoryFunction;
    readonly RequiredArgExistAfterOptionalArg: ValidationErrorFactoryFunction;
    readonly EmptySample: ValidationErrorFactoryFunction;
    readonly DuplicateProperty: ValidationErrorFactoryFunction;
    readonly DuplicatePropertyRedeclared: ValidationErrorFactoryFunction;
    readonly IdentifierExpected: ValidationErrorFactoryFunction;
    readonly VarArgsMustBeLast: ValidationErrorFactoryFunction;
    readonly UndefinedCatalogUndefinedType: ValidationErrorFactoryFunction;
    readonly UndefinedCatalogCustomType: ValidationErrorFactoryFunction;
    readonly InvalidNamespace: ValidationErrorFactoryFunction;
    readonly InvalidNameDeclaration: ValidationErrorFactoryFunction;
    readonly EmptyUtteranceSet: ValidationErrorFactoryFunction;
    readonly InvalidUtteranceItem: ValidationErrorFactoryFunction;
    readonly InvalidUtteranceArg: ValidationErrorFactoryFunction;
    readonly EmptyDialog: ValidationErrorFactoryFunction;
    readonly SkillNotFound: ValidationErrorFactoryFunction;
    readonly AmbiguousNameReference: ValidationErrorFactoryFunction;
    readonly InvalidSkillExpression: ValidationErrorFactoryFunction;
    readonly CouldNotInferAnonymousType: ValidationErrorFactoryFunction;
    readonly CouldNotInferGenericArguments: ValidationErrorFactoryFunction;
    readonly PositionalArgumentAfterName: ValidationErrorFactoryFunction;
    readonly InvalidSkillCallContextNameDeclaration: ValidationErrorFactoryFunction;
    readonly InvalidSkillCallContextModuleScope: ValidationErrorFactoryFunction;
    readonly NoDialogsInSkill: ValidationErrorFactoryFunction;
    readonly NoDeployableDialog: ValidationErrorFactoryFunction;
    readonly NoDeployableDialogForLocale: ValidationErrorFactoryFunction;
    readonly TooManySkillDeclarations: ValidationErrorFactoryFunction;
    readonly InvalidReferableAnnotation: ValidationErrorFactoryFunction;
    readonly NameNotFoundInUtteranceValue: ValidationErrorFactoryFunction;
    readonly ArgumentMustReferToActionArgument: ValidationErrorFactoryFunction;
    readonly ArgumentMustReferToActionArgumentWithUtteranceType: ValidationErrorFactoryFunction;
    readonly ExpectedNonEmptyArgumentList: ValidationErrorFactoryFunction;
    readonly UtteranceEventTypeMustReferToWrapper: ValidationErrorFactoryFunction;
    readonly ExpectedEmptyRequestActType: ValidationErrorFactoryFunction;
    readonly InvokedActionNotOffered: ValidationErrorFactoryFunction;
    readonly EmptyUtteranceString: ValidationErrorFactoryFunction;
    readonly InvalidBuiltinCatalog: ValidationErrorFactoryFunction;
    readonly InvalidCatalogTypeParameter: ValidationErrorFactoryFunction;
    readonly CatalogTypeMustBeString: ValidationErrorFactoryFunction;
    readonly InvalidCatalogSubType: ValidationErrorFactoryFunction;
    readonly InvalidDialogCall: ValidationErrorFactoryFunction;
    readonly ComplexUtteranceType: ValidationErrorFactoryFunction;
    readonly ACDLCompilationSkipped: ValidationErrorFactoryFunction;
    readonly NonMatchingInteractionModelIntents: ValidationErrorFactoryFunction;
    readonly InvalidApiReturnType: ValidationErrorFactoryFunction;
    readonly ApiArgumentWithNoCatalog: ValidationErrorFactoryFunction;
    readonly InvalidNamespacesProvidedToGetAllNames: ValidationErrorFactoryFunction;
    readonly IntentNotGenerated: ValidationErrorFactoryFunction;
    readonly InvalidExpressionAfterDelegation: ValidationErrorFactoryFunction;
    readonly InvalidChainedDirective: ValidationErrorFactoryFunction;
    readonly InvalidDelegationSlots: ValidationErrorFactoryFunction;
    readonly InvalidDelegationActionConfirmation: ValidationErrorFactoryFunction;
    readonly InvalidDelegationArgumentType: ValidationErrorFactoryFunction;
    readonly InvalidCatalogCallContext: ValidationErrorFactoryFunction;
};
export type ValidationErrorFactoryEntry = (typeof ValidationErrorFactory)[keyof typeof ValidationErrorFactory];
export {};
//# sourceMappingURL=error-factory.d.ts.map